#if !defined (__KQML_H__)

#define __KQML_H__

#include <stdio.h> 	/* FILE */

#include <kqmlmsg.h>

typedef struct kqml_kmsg_list {
   	kqml_message *kmsg;
   	struct kqml_kmsg_list *next;
   	struct kqml_kmsg_list *prev;
   } KQML_KMSG_LIST;

#include <kqmlwrapper.h>
#include <kqmldbg.h>

#define KQML_SEND_TIMEOUT -1

#define KQML_HANDLER_ERROR (KQML_KMSG_LIST *)-1
#define KQML_HANDLER_NONE (KQML_KMSG_LIST *)0

/*
 * A handler is a function which accepts a kqml message and
 * processes it. It returns a void *. The type of the reply is
 * determined by reply_type.
 */
typedef void *(*handler)(char *content, kqml_message *msg,
	KQML_REPLY_TYPE *reply_type);

typedef int (*ans_handler)(char *agent, char **name, char **protocol,
	char **addr_info);

typedef enum { 
	KQML_PARSE_OK = 0, KQML_PARSE_EOF, KQML_PARSE_NOTHING, KQML_PARSE_TIMEOUT, 
		KQML_PARSE_INCOMPLETE, KQML_PARSE_SYNTAX, KQML_PARSE_MEMORY, 
		KQML_PARSE_SEMANTIC 
	} kqml_parse_error;

#define KQML_NOT_AN_ANS (ans_handler)0

#ifdef __cplusplus
extern "C" {
#endif

extern char *kqml_agent_name, *kqml_ans_name, *kqml_protocol, *kqml_addr_info;

extern int kqml_initialize (char *agent_name, ans_handler ans_lookup);
extern int kqml_register (handler hfunction, char *perfname, int blocking);

extern KQML_KMSG_LIST *kqml_build_kmsg_list (KQML_KMSG_LIST **reply_msg,
	kqml_message *kmsg);
extern void kqml_print_kmsg_list (FILE *stream, KQML_KMSG_LIST *kmsg_list);
extern void kqml_free_kmsg_list (KQML_KMSG_LIST *kmsg_list);

extern int kqml_parse_fd (int fd, kqml_message **msg);
extern int kqml_parse_stream (FILE *stream, kqml_message **msg);
extern int kqml_parse_buffer (char *buffer, int len, int *processed_len, 
	kqml_message **msg);

extern kqml_message *kqml_build_msgv (char *performative, char *content, 
	char *rec_agent, char *reply_with, char *field[], char *vlaue[]);
extern kqml_message *kqml_build_msg (char *performative, char *content, 
	char *rec_agent, char *reply_with, ...);
extern void kqml_free_msg (kqml_message *msg);

extern int kqml_get_field_count (kqml_message *msg);
extern char *kqml_get_performative (kqml_message *msg);
extern char *kqml_get_field (char *field_name, kqml_message *msg, 
	int *value_len, kqml_expression_type *value_type);
extern char *kqml_get_ith_field (int field_no, kqml_message *msg,
	char **field_name, char **value, int *value_len, 
	kqml_expression_type *value_type);

extern int kqml_put_performative (kqml_message *msg, char *performative);
extern int kqml_put_field (char *field_name, kqml_message *msg, char *value,
	int value_len);
extern int kqml_put_embedded_msg (char *field_name, kqml_message *msg, 
	kqml_message *emsg);

extern int kqml_send (int timeout, kqml_message *msg, kqml_message **reply);
extern int kqml_send_msg (int timeout, char *perf, char *content, 
	char *rec_agent, char *reply_with, kqml_message **reply, ...);
extern int kqml_send_msgv (int timeout, char *perf, char *content, 
	char *rec_agent, char *reply_with, kqml_message **reply, 
	char *field[], char *value[]);

/** Added to support multitasking in LISP interface -- Anupama **/
extern int _kqml_send_xdr_msg (kqml_message *);
extern void kqml_prep_outmsg (kqml_message *, kqml_message *);
/****************************************************************/

extern int kqml_deliver (kqml_message *msg);
extern int kqml_deliver_msg (char *perf, char *content, ...);
extern int kqml_deliver_msgv (char *perf, char *content, char *field[], 
	char *value[]);

extern void kqml_print_msg (FILE *stream, kqml_message *msg);
extern char *kqml_sprint_msg (char *buffer, int *size, kqml_message *msg);

extern int kqml_critical_section_start ();
extern int kqml_critical_section_end ();

extern void kqml_timings_start ();
extern void kqml_timings_stop ();
extern int kqml_timings ();

extern void kqml_logging_start ();
extern void kqml_logging_stop ();
extern int kqml_logging ();

extern int kqml_stricmp (char *s1, char *s2);
extern int kqml_strincmp (char *s1, char *s2, int n);
extern int kqml_match_agent_names (char *s1, char *s2);
extern int kqml_digit_count (long number);
extern int kqml_digit_to_str (long number, char *str);
extern int kqml_Udigit_count (unsigned long number);
extern int kqml_Udigit_to_str (unsigned long number, char *str);

#ifdef __cplusplus
}
#endif

#endif /* __KQML_H__ */
