/*
 * dbgmsg.c - standard debugging routines
 *
 *
 */
#include <kqml/kqmlconf.h>

#include	<stdio.h>
#include	<stdarg.h>
#include <sys/types.h>
#include	<time.h>
#include	<errno.h>
#include	<unistd.h>
#include	<fcntl.h>

#include <kqml.h>
#include <kqml/kqmllib.h>

static FILE *dbgfile = stderr;
static char *app_name = "uninitialized";

int  _kqml_dbg_level = KQML_DBG_ALWAYS;

#ifdef __KQML_MACH_SUN__
extern char	*sys_errlist[];
#endif

static void dbg_lock (int);

void
_kqml_init_debugging (char *name, FILE *file, int dbg_level)
{
	if (file)
	{
		dbgfile = file;
	}
	_kqml_dbg_level = dbg_level;
	app_name = strdup(name);
	if (app_name == NULL)
	{
		fprintf(stderr, "Error in allocating space for the appln name 
				in Debug\n");
	}
}


/*
 * dbgmsg1 is a VARARGS routine, which is usually called using the macro
 * dbgmsg in the following way:
 *
 * dbgmsg(level)(errnoflg, format-string, args);
 * 
 * Note that it is the macro which takes care of checking the value of
 * the global "dbglevel", not dbgmsg1.
 *
 */

void
_kqml_dbg_msg1 (char *fmt, ...)
{
	va_list args;
	char buf[7];
	long now = time (NULL);
	struct tm *tim;
    
	va_start (args, fmt);
	tim = localtime (&now);
	strftime (buf, sizeof(buf), "%H%M%S", tim);
    
	/* dbg_lock (1); */

	fprintf (dbgfile, "%6s %s: ", buf, app_name);
	vfprintf (dbgfile, fmt, args);
	fprintf (dbgfile, "\n");
	fflush (dbgfile);

	/* dbg_lock (0); */
	
	va_end (args);
}

void
_kqml_dbg_kmsg1 (kqml_message *kmsg)
{
	/* dbg_lock (1); */

	kqml_print_msg (dbgfile, kmsg);
	fflush (dbgfile);

	/* dbg_lock (0); */
}

void
_kqml_dbglevelset (int level)
{
	_kqml_dbg_level = level;
}

/* LOCKING is disabled. It introduces upto 3 seconds of delay, which
	is precious and we can live with a slightly corrupted debug msg,
	which anyway does not happen often */

static void
dbg_lock (int lock)
{
	int fd;
	static struct flock dbg_flock   = {F_WRLCK, SEEK_SET, 0, 0, 0};
	static struct flock dbg_funlock = {F_UNLCK, SEEK_SET, 0, 0, 0};

	/* for now turn off the locking mechanism */
	return;
	
	fd = fileno (dbgfile);
	if (isatty(fd))
		return;
    
	if (lock)
	{
		while (fcntl (fd, F_SETLKW, &dbg_flock) <= -1) 
		{
			if (errno != EINTR)
			{
				break;
			}
		}
	}
	else
	{
		while (fcntl (fd, F_SETLK, &dbg_funlock) <= -1) 
		{
			if (errno != EINTR)
			{
				break;
			}
		}
	}
}
