
// Author : Gregory R. Olsen 
//          Center for Design Research, Stanford University
//          560 Panama Street
//          Stanford, CA 94305-2232
//          olsen@cs.stanford.edu

// Copyright (c) 1993 Stanford University. All rights reserved.

/*
 * Permission is hereby granted, without written agreement and without
 * license, to use, copy, modify, and distribute this software and its 
 * documentation for non-commercial use, provided that the above 
 * copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL STANFORD UNIVERSITY BE LIABLE FOR ANY SPECIAL, 
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
 * WHETHER OR NOT ADVISEDOF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY
 * OF LIABILITY, ARISING  OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "KIFClass.h"
#include "KIFGrammar.tab.h"
#define MAXMESLEN 1024
  extern int yydebug;

extern expression *kif_parse(int, const char *);
void printparse (expression *x);


static int get_line2(FILE *id,char *line,int max)
{
  int c,i;

  i=0;
  while ((c=getc(id)) != '\n' && c != EOF)
    if (i<max-1)
      line[i++]=c;
  line[i]='\0';
  if (c==EOF)
    return EOF;
  else
    return i;
}

main(int argc, char **argv)
{
  char buffer[MAXMESLEN];
  char *tmp;
  int c;
  expression *kif_obj;
//  yydebug = 1;
  
				/* assume this is a form */
  while (1) 
    {
      
      fprintf(stdout,"\nEnter KIF Form:  ");
      get_line2(stdin,buffer,MAXMESLEN);
      
      kif_obj = kif_parse(0,buffer);
      
      if (kif_obj != NULL)
	tmp = kif_obj->kifstring();
      else
	tmp = strdup("Unparseable");
      
      fprintf(stdout,"\nTest Print: <%s> %s\n\n",typeName(kif_obj),tmp);
      printparse (kif_obj);
      fflush(stdout);
      
      delete kif_obj;
      delete tmp;
    }
  
}



void printparse (expression *x)
{
  expression *k, *v;
  sequence *s;
  
  if (!x) return;
  
  printf ("Performative: %s\n", x->relation()->kifstring());
  k = x->terms()->hd();
  v = x->terms()->tl()->hd();
  s = x->terms()->tl()->tl();
  while (strcmp ("",k->kifstring()) != 0)
    {
      printf ("Key: %s\nValue: %s\n\n", k->kifstring(), v->kifstring());
      k = s->hd();
      v = s->tl()->hd();
      s = s->tl()->tl();
    }
  return;
}
